/*
 * Simulator of MCS51 (brk.cc)
 *
 * Copyright (C) 1997,97 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of MCS51 simulator: s51.

S51 is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

S51 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with S51; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#include "ddconfig.h"

#include "pobj.h"
#include "brkcl.h"


/*
 * Base object of breakpoints
 *____________________________________________________________________________
 *
 */

brkcl::brkcl(uint Iaddr,
	     enum brk_perm Iperm,
	     int  Ihit):
  TObject()
{
  addr = Iaddr;
  perm = Iperm;
  hit  = Ihit;
  cnt  = Ihit;
}

brkcl::~brkcl(void)
{}


/*
 * FETCH type of breakpoint
 *____________________________________________________________________________
 *
 */

brk_fetch::brk_fetch(uint Iaddr,
		     enum brk_perm Iperm,
		     int  Ihit):
  brkcl(Iaddr, Iperm, Ihit)
{
  code = 0;
}

enum brk_type
brk_fetch::type(void)
{
  return(brkFETCH);
}


/*
 * Base of EVENT type of breakpoints
 *____________________________________________________________________________
 *
 */

brk_ev::brk_ev(uint Iaddr,
	       enum brk_perm Iperm,
	       int  Ihit,
	       enum brk_event Ievent,
	       const char *Iid):
  brkcl(Iaddr, Iperm, Ihit)
{
  event= Ievent;
  id   = Iid;
}

enum brk_type
brk_ev::type(void)
{
  return(brkEVENT);
}

bool
brk_ev::match(struct event_rec *ev)
{
  return(FALSE);
}


/*
 * WRITE IRAM type of EVENT breakpoints
 *____________________________________________________________________________
 *
 */

brk_wi::brk_wi(uint Iaddr,
	       enum brk_perm Iperm,
	       int  Ihit):
  brk_ev(Iaddr, Iperm, Ihit, brkWIRAM, "wi")
{
}

bool
brk_wi::match(struct event_rec *ev)
{
  return(ev->wi == addr);
}


/*
 * READ IRAM type of EVENT breakpoints
 *____________________________________________________________________________
 *
 */

brk_ri::brk_ri(uint Iaddr,
	       enum brk_perm Iperm,
	       int  Ihit):
  brk_ev(Iaddr, Iperm, Ihit, brkRIRAM, "ri")
{
}

bool
brk_ri::match(struct event_rec *ev)
{
  return(ev->ri == addr);
}


/*
 * WRITE XRAM type of EVENT breakpoints
 *____________________________________________________________________________
 *
 */

brk_wx::brk_wx(uint Iaddr,
	       enum brk_perm Iperm,
	       int  Ihit):
  brk_ev(Iaddr, Iperm, Ihit, brkWXRAM, "wx")
{
}

bool
brk_wx::match(struct event_rec *ev)
{
  return(ev->wx == addr);
}


/*
 * READ XRAM type of EVENT breakpoints
 *____________________________________________________________________________
 *
 */

brk_rx::brk_rx(uint Iaddr,
	       enum brk_perm Iperm,
	       int  Ihit):
  brk_ev(Iaddr, Iperm, Ihit, brkRXRAM, "rx")
{
}

bool
brk_rx::match(struct event_rec *ev)
{
  return(ev->rx == addr);
}


/*
 * WRITE SFR type of EVENT breakpoints
 *____________________________________________________________________________
 *
 */

brk_ws::brk_ws(uint Iaddr,
	       enum brk_perm Iperm,
	       int  Ihit):
  brk_ev(Iaddr, Iperm, Ihit, brkWSFR, "ws")
{
}

bool
brk_ws::match(struct event_rec *ev)
{
  return(ev->ws == addr);
}


/*
 * READ SFR type of EVENT breakpoints
 *____________________________________________________________________________
 *
 */

brk_rs::brk_rs(uint Iaddr,
	       enum brk_perm Iperm,
	       int  Ihit):
  brk_ev(Iaddr, Iperm, Ihit, brkRSFR, "rs")
{
}

bool
brk_rs::match(struct event_rec *ev)
{
  return(ev->rs == addr);
}


/*
 * READ CODE type of EVENT breakpoints
 *____________________________________________________________________________
 *
 */

brk_rc::brk_rc(uint Iaddr,
	       enum brk_perm Iperm,
	       int  Ihit):
  brk_ev(Iaddr, Iperm, Ihit, brkRCODE, "rc")
{
}

bool
brk_rc::match(struct event_rec *ev)
{
  return(ev->rc == addr);
}


/*
 * Making collection of break-points
 *____________________________________________________________________________
 *
 */

brk_coll::brk_coll(TIndex ALimit, TIndex ADelta):
  TSortedCollection(ALimit, ADelta)
{
}


/*
 * Collection of break-points
 *____________________________________________________________________________
 *
 * This is a sorted collection, sorted by addr field of brk items.
 *
 */

void *
brk_coll::KeyOf(void *Item)
{
  return((void *)&(((class brkcl *)(Item))->addr));
}


/*
 * Comparing key fields of two brk items
 *____________________________________________________________________________
 *
 */

int
brk_coll::Compare(void *Key1, void *Key2)
{
  int k1, k2;

  k1= *(int *)Key1;
  k2= *(int *)Key2;

  if (k1 == k2)
    return(0);
  else
    if (k1 < k2)
      return(-1);
    else
      return(+1);
}


/*
 * Checking if there is an event breakpoint for the specified event
 *____________________________________________________________________________
 *
 */

bool
brk_coll::there_is_event(enum brk_event ev)
{
  class brkcl *b;
  int   i;

  for (i= 0; i < Count; i++)
    {
      b= (class brkcl *)At(i);
      if (b->type() == brkEVENT &&
	  ((class brk_ev *)b)->event == ev)
	return(TRUE);
    }
  return(FALSE);
}


/* End of brk.cc */
