/*
 * Simulator of MCS51 (cmd.cc)
 *
 * Copyright (C) 1997,97 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of MCS51 simulator: s51.

S51 is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

S51 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with S51; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#include "ddconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "i_string.h"

#include "uccl.h"
#include "globals.h"

#include "dump.h"
#include "go.h"
#include "cmd_brk.h"
#include "set.h"
#include "where.h"
#include "option.h"
#include "show.h"


static char *last_command= NULL;

/* Type of entries in command table */

struct cmd_entry
{
  // Name of the command
  char *name;
  // Function to execute
  bool (*func)(char *cmd, class t_uc *uc);
  // Command can be repeated by empty command
  bool can_repeat;
  // Help about the command
  char *help;
};

/* Table is defined at the end of this file */

static struct cmd_entry cmd_table[];


/*
 * HELP
 *____________________________________________________________________________
 *
 */

static bool
cmd_help(char *cmd, class t_uc *uc)
{
  int i;

  i= 0;
  while (cmd_table[i].name != NULL)
    {
      if (cmd_table[i].help != NULL)
	fprintf(uc->cmd_out, "%s\n", cmd_table[i].help);
      i++;
    }
  return(FALSE);
}


/*
 * QUIT
 *____________________________________________________________________________
 *
 */

static bool
cmd_quit(char *cmd, class t_uc *uc)
{
  return(TRUE);
}


/*
 * LOAD file
 *____________________________________________________________________________
 *
 */

static bool
cmd_load(char *cmd, class t_uc *uc)
{
  char *name;
  long l;

  if ((name= strtok(NULL, delimiters)) == NULL)
    fprintf(uc->cmd_out, "File name has not given\n");
  else
    if ((l= uc->read_hex_file(name)) >= 0)
      fprintf(uc->cmd_out, "%ld bytes read from %s\n", l, name);
  return(FALSE);
}


/*
 * DOWNLOAD
 *____________________________________________________________________________
 *
 */

static bool
cmd_download(char *cmd, class t_uc *uc)
{
  long l;
  char c;

  if ((l= uc->read_hex_file(NULL)) >= 0)
    fprintf(uc->cmd_out, "%ld bytes read\n", l);
  //while ((c= getc(uc->cmd_in)) != '\n') ;
  return(FALSE);
}


/*
 * RESET
 *____________________________________________________________________________
 *
 */

static bool
cmd_reset(char *cmd, class t_uc *uc)
{
  uc->reset();
  return(FALSE);
}


/*
 * GET OPTION [opt]
 *____________________________________________________________________________
 *
 */

static bool
cmd_get_option(char *cmd, class t_uc *uc)
{
  char *s;
  int i;
  class option_base *o;

  s= strtok(NULL, delimiters);
  for (i= 0; i < uc->options->GetCount(); i++)
    {
      o= (class option_base *)(uc->options->At(i));
      if (!s ||
	  !strcmp(s, o->id))
	{
	  fprintf(uc->cmd_out, "%s ", o->id);
	  o->print(uc->cmd_out);
	  fprintf(uc->cmd_out, " %s\n", o->help);
	}
    }
  return(FALSE);
}


/*
 * SET OPTION opt value
 *____________________________________________________________________________
 *
 */

static bool
cmd_set_option(char *cmd, class t_uc *uc)
{
  char *s, *id;
  int i;
  class option_base *o;

  if ((id= strtok(NULL, delimiters)) == NULL)
    {
      fprintf(uc->cmd_out, "Name of option has not given.\n");
      return(FALSE);
    }
  if ((s= strtok(NULL, delimiters)) == NULL)
    {
      fprintf(uc->cmd_out, "Value has not given.\n");
      return(FALSE);
    }
  for (i= 0; i < uc->options->GetCount(); i++)
    {
      o= (class option_base *)(uc->options->At(i));
      if (!strcmp(id, o->id))
	{
	  o->set_value(s);
	  break;
	}
    }
  return(FALSE);
}


/*
 * Reading a command from standard input
 *____________________________________________________________________________
 *
 */

static char *
read_cmd(FILE *f)
{
  char *s= NULL, *p;
  size_t n= 30;

#ifdef HAVE_GETLINE
  if (getline(&s, 0, f) < 0)
    return(NULL);
#else

# ifdef HAVE_GETDELIM
  s= (char *)malloc(n);
  if (getdelim(&s, &n, '\n', f) < 0)
    {
      free(s);
      return(NULL);
    }
# else

#  ifdef HAVE_FGETS
  s= (char *)malloc(300);
  if (fgets(s, 300, f) == NULL)
    {
      free(s);
      return(NULL);
    }
#  endif
# endif
#endif
  s[strlen(s)-1]= '\0';
  return(s);
}


/*
 * Interpreting the specified command
 *____________________________________________________________________________
 *
 * This function returns TRUE if interpreter can quit.
 *
 */

static bool
interpret(char *cmd, class t_uc *uc)
{
  int i;
  char *c, *s;
  bool repeat= FALSE, retval= FALSE;
  
  if (*cmd)
    {
      c= strdup(cmd);
      if (last_command)
	free(last_command);
      last_command= strdup(cmd);
    }
  else
    if (last_command == NULL)
      return(FALSE);
    else
      {
	c= strdup(last_command);
	repeat= TRUE;
      }
  i= 0;
  s= strtok(c, delimiters);
  while ((cmd_table[i].name != NULL) &&
	 /*(strstr(c, cmd_table[i].name) != c)*/
	 (strcmp(s, cmd_table[i].name) != 0))
    i++;
  if (cmd_table[i].name == NULL)
    {
      fprintf(uc->cmd_out, "Unknown command.\n");
      if (last_command)
	{
	  free(last_command);
	  last_command= NULL;
	}
    }
  else
    {
      if (!repeat)
	retval= cmd_table[i].func(c, uc);
      else
	if (cmd_table[i].can_repeat)
	  retval= cmd_table[i].func(c, uc);
    }
  free(c);
  return(retval);
}


/*
 * Printing out the prompt
 *____________________________________________________________________________
 *
 */

void
prompt(FILE *f)
{
  if (prompt_str)
    fprintf(f, "%s", prompt_str);
  else
    putc('\0', f);
  fflush(f);
}


/*
 * Command interpreter of s51
 *____________________________________________________________________________
 *
 */

int
t_uc::do_cmd(void)
{
  char *cmd;
  bool done;
  
  done= FALSE;
  while (!done)
    {
      prompt(cmd_out);
      if ((cmd= read_cmd(cmd_in)) != NULL)
	{
	  done= interpret(cmd, this);
	  free(cmd);
	}
      else
	{
	  fprintf(cmd_out, "\n");
	  done= TRUE;
	}
    }
  if (last_command)
    free(last_command);
  return(0);
}


/*
 * Table of commands and their names
 *____________________________________________________________________________
 *
 */

static struct cmd_entry cmd_table[]=
{
  {"l" , cmd_load, FALSE,
   "l file             Load (intel hexa) file"},

  {"dl", cmd_download, FALSE,
   "dl,do              Download (intel hex) data"},
  {"do", cmd_download, FALSE, NULL},

  {"g" , cmd_go, FALSE,
   "g [start [stop]]   Go"},

  {"pc", cmd_pc, FALSE,
   "pc [address]       Get/set content of PC"},

  {"s" , cmd_step, TRUE,
   "s [step]           Step"},

  {"n" , cmd_next, TRUE,
   "n [step]           Next"},

  {"bs", cmd_brk_setf, FALSE,
   "bs f|d addr [hit]  Set FETCH Breakpoint"},

  {"bse", cmd_brk_sete, FALSE,
   "bse wi|ri|wx|rx|ws|rs|rc f|d addr [hit]\n"
   "                   Set EVENT Breakpoint"},

  {"bd", cmd_brk_delf, FALSE,
   "bd addr            Delete FETCH Breakpoint"},

  {"bde", cmd_brk_dele, FALSE,
   "bde wi|ri|wx|rx|ws|rs|rc addr\n"
   "                   Delete EVENT Breakpoint"},

  {"ba", cmd_brk_delall, FALSE,
   "ba                 Delete all breakpoints"},

  {"bl", cmd_brk_lst, FALSE,
   "bl                 List Breakpoints"},

  {"dis", cmd_disass, TRUE,
   "dis [start [offset [lines]]]\n"
   "                   Disassemble code"},

  {"dc", cmd_dump_code, TRUE,
   "dc [start [stop]]  Dump code in disass form"},

  {"dch", cmd_dump_code_hex, TRUE,
   "dch [start [stop]] Dump code in hex form"},

  {"di", cmd_dump_iram, TRUE,
   "di [start [stop]]  Dump Internal RAM"},

  {"dx", cmd_dump_xram, TRUE,
   "dx [start [stop]]  Dump External RAM"},

  {"dr", cmd_dump_regs, TRUE,
   "dr                 Dump registers"},

  {"dp", cmd_dump_port, FALSE,
   "dp                 Dump ports"},

  {"ds", cmd_dump_sfr, FALSE,
   "ds [addr...]       Dump SFR"},

  {"db", cmd_dump_bit, TRUE,
   "db addr...         Dump bit"},

  {"si", cmd_set_iram, FALSE,
   "si addr data...    Set Internal RAM"},

  {"sx", cmd_set_xram, FALSE,
   "sx addr data...    Set External RAM"},

  {"sc", cmd_set_code, FALSE,
   "sc addr data...    Set code (ROM)"},
  
  {"ss", cmd_set_sfr, FALSE,
   "ss addr data...    Set SFR area"},

  {"sb", cmd_set_bit, FALSE,
   "sb addr data...    Set bit"},

  {"sp", cmd_set_port, FALSE,
   "sp port data       Set port pins"},

  {"fi", cmd_fill_iram, FALSE,
   "fi start stop data Fill IRAM area with `data'"},

  {"fx", cmd_fill_xram, FALSE,
   "fx start stop data Fill XRAM area with `data'"},
       
  {"fs", cmd_fill_sfr, FALSE,
   "fs start stop data Fill SFR area with `data'"},

  {"fc", cmd_fill_code, FALSE,
   "fc start stop data Fill ROM area with `data'"},

  {"res", cmd_reset, FALSE,
   "res                Reset"},

  {"wi", cmd_where_iram, FALSE,
   "wi,Wi string       Search for `string' in IRAM (Wi case sensitive)"},
  {"Wi", cmd_Where_iram, FALSE, NULL},

  {"wx", cmd_where_xram, FALSE,
   "wx,Wx string       Search for `string' in XRAM (Wx case sensitive)"},
  {"Wx", cmd_Where_xram, FALSE, NULL},

  {"wc", cmd_where_code, FALSE,
   "wc,Wc string       Search for `string' in ROM (Wc case sensitive)"},
  {"Wc", cmd_Where_code, FALSE, NULL},

  {"sopt", cmd_set_option, FALSE,
   "sopt opt value     Set value of option"},

  {"gopt", cmd_get_option, FALSE,
   "gopt [opt]         Get value of option(s)"},

  {"q" , cmd_quit, FALSE,
   "q                  Quit"},

  {"show", cmd_show, FALSE,
   "show c|w           Show licensing information"},

  {"h" , cmd_help, FALSE,
   "h,?                Help about commands"},
  {"?" , cmd_help, FALSE, NULL},

  {NULL, NULL}
};

/* End of cmd.cc */
